// NoverK.java by Peter Alexay, 2003-04-28 and 2004-03-19

import java.math.*;

public class NoverK
{
  public static final long MAXN=1200;
  public static final BigInteger ten=BigInteger.valueOf(10L);
  
  public static BigInteger factorial(long begin,long end)
  {
    BigInteger counter=BigInteger.valueOf(begin);
    BigInteger result=BigInteger.ONE;
    for (long i=begin; i<=end; i++)
    {
      result=result.multiply(counter);
      counter=counter.add(BigInteger.ONE);
    }
    return result;
  } // factorial()
  
  public static BigInteger noverk(long n,long k)
  {
    if (k>n) return BigInteger.ZERO;
    BigInteger szamlalo=factorial(k+1,n);
    BigInteger nevezo=factorial(1,n-k);
    BigInteger result=szamlalo.divide(nevezo);
    return result;
  } // noverk
  
  
  public static BigInteger tenExponent(int exponent)
  {
    return ten.pow(exponent);
  } // tenExponent()
  
  
  public static void do829test()
  {
    boolean found=false;
    BigInteger res=null,difference=null,relError=null;
    String resString=null;
    BigInteger tenexponent_res=null;
    //String tenexponent_resString=null;
    int exponent=0;

    for (long n=5; n<=MAXN; n+=5)
    {
      exponent+=3; // exponent=0.6*n
      found=false;
      res=noverk(2*n,n);
      resString=res.toString();
      if (resString.length()<5)
      {
        //tenexponent_res=tenExponent(exponent);
        //tenexponent_resString=tenexponent_res.toString();
//        System.out.print("n="+n+", "+(2*n)+" over "+n+"="+resString+".");
//        System.out.println("   10^(0.6*n)="+tenexponent_resString);
        System.out.println("\nn="+n+", calculating "+(2*n)+" over "+n+":");
        System.out.println("   combinatorical result: "+resString);
        System.out.println("   approximative result : 1.00*10^"+exponent);
      }
      else
      {
        String mant=resString.substring(0,1)+"."+resString.substring(1,3);
        int exp=resString.length()-1;
        tenexponent_res=tenExponent(exponent);
        difference=tenexponent_res.subtract(res).abs();
        String diffString=difference.toString();
        if (mant.equals("1.00") && (exp==exponent))
        {
          System.out.print("*******************"); // approx. result = combinatorical result (in our range)
        }
        System.out.println("\nn="+n+", calculating "+(2*n)+" over "+n+":");
        System.out.println("   combinatorical result: "+mant+"*10^"+exp);
        System.out.println("   approximative result : 1.00*10^"+exponent);
//        System.out.print("   n="+n+", "+(2*n)+" over "+n+"="+resString+".");
//        System.out.println("  10^(0.6*n)="+tenexponent_resString);
        String diffMant=diffString.substring(0,1)+"."+diffString.substring(1,3);
        System.out.println("   absolute error       : "+diffMant+"*10^"+(diffString.length()-1));
        relError=difference.divide(res);
        System.out.println("   relative error       : "+relError.toString());
        relError=difference.divide(res.divide(BigInteger.valueOf(10L)));
        System.out.println("   relative error*10    : "+relError.toString());
      } // else
    } // for
  } // do829test()
  
  public static void usage()
  {
    System.err.println("Usage: java NoverK [n k]");
    System.exit(1);
  }
  
  public static void main(String[] args)
  {
    if (args.length!=2) do829test();
    else
    {
      long n=0,k=0;
    
      try
      {
        n=Long.parseLong(args[0]);
        k=Long.parseLong(args[1]);
      }
      catch (Exception e)
      {
        usage();
      }
    
      System.out.println("Calculating "+n+" over "+k+"...");
      BigInteger result=noverk(n,k);
      System.out.println("  Result: "+result.toString());
    } // else
  } // main()
} // class NoverK
