/*
   NoverKCanvas by Peter Alexay, 2004-03-19

   Draws the relative error (difference) between (2n over n) and the approximated value.
   The function has a minimum at n=830.
   For greater values of n, the relative error remains nearly constant.
*/

import java.awt.*;
import java.awt.image.*;
import java.math.*;

public class NoverKCanvas extends Canvas
{
    public static final int width=640;
    public static final int height=512;
    private static final long MAXN=2000L;
    private static final long RelErrorScale=20L;
    
    int[] imgData;
    ColorModel cm;
    Image img;
    MemoryImageSource misource;

    public NoverKCanvas()
    {
        cm = new DirectColorModel(24, 0x00FF0000, 0x0000FF00, 0x000000FF);
        imgData = new int[width*height];
        misource = new MemoryImageSource(width, height, cm, imgData, 0, width);
        misource.setAnimated(true);
        misource.setFullBufferUpdates(true);
        img = createImage(misource);
    }
    
    public void clear()
    {
      for (int i=0; i<width*height; i++)
	    {
        imgData[i]=0;
	    }
    }

    public void setPixel(int x, int y, Color c)
    {
      if ( x > 0 && x < width && y > 0 && y < height )
	      imgData[y*width + x] = (c.getRed() << 16) | (c.getGreen() << 8) | c.getBlue();
    }
    	
    public void paint(Graphics g)
    {
        misource.newPixels(0,0,img.getWidth(null),img.getHeight(null));
    
        g.drawImage(img, 0, 0, null);
    }

    public void computeFunction()
    {
      BigInteger res=null,difference=null,relError=null;
      BigInteger tenexponent_res=null;
      //String tenexponent_resString=null;
      int exponent=0;
      int xcoord=1,ycoord;
  
      for (long n=5; n<=MAXN; n+=5)
      {
        exponent+=3; // exponent=0.6*n
        res=NoverK.noverk(2*n,n);
        tenexponent_res=NoverK.tenExponent(exponent);
        difference=tenexponent_res.subtract(res).abs();
        relError=difference.divide(res.divide(BigInteger.valueOf(RelErrorScale)));
        ycoord=relError.intValue();
        //System.out.println("xcoord="+xcoord+", ycoord="+ycoord);
        setPixel(xcoord,height-ycoord-1,Color.red);
        xcoord++;
        if (xcoord>width) break;
        if (xcoord%20==0) drawFunction(); // draw the function part in order to give feedback
      } // for
    }

    public void drawFunction()
    {
      Graphics g = getGraphics();
      paint(g);
      g.dispose();
    }

} // class NoverKCanvas
